
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "c-utils-common.h"

static const int buffer_size = 2*1024;

/**
   Is there another way to detect we are under valgrind?
 */
static bool
using_valgrind(void)
{
  // User must set VALGRIND to get this to work
  char* s = getenv("VALGRIND");
  if (s != NULL && strlen(s) > 0)
    return true;
  return false;
}

void
valgrind_assert_failed(const char* file, int line)
{
  printf("valgrind_assert(): failed: %s:%d\n", file, line);
  if (using_valgrind())
  {
    printf("valgrind_assert(): inducing memory fault...\n");
    // This will give us more information from valgrind
    puts((char*) 1);
  }
  abort();
}

void
valgrind_assert_failed_msg(const char* file, int line,
                           const char* format, ...)
{
  printf("valgrind_assert(): failed: %s:%d\n", file, line);
  char buffer[buffer_size];
  int count = 0;
  char* p = &buffer[0];
  va_list ap;
  va_start(ap, format);
  count += sprintf(p, "valgrind_assert(): ");
  count += vsnprintf(buffer+count, buffer_size-count, format, ap);
  va_end(ap);
  printf("%s\n", buffer);
  fflush(NULL);
  if (using_valgrind())
  {
    printf("valgrind_assert(): inducing memory fault...\n");
    // This will give us more information from valgrind
    puts((char*) 1);
  }
  abort();
}
