
#ifndef C_UTILS_COMMON_H
#define C_UTILS_COMMON_H

/** Called when the valgrind_assert() condition fails */
void valgrind_assert_failed(const char *file, int line);

/** Called when the valgrind_assert_msg() condition fails */
void valgrind_assert_failed_msg(const char *file, int line,
                           const char* format, ...);

/**
   VALGRIND_ASSERT
   Substitute for assert(): provide stack trace via valgrind
   If not running under valgrind, works like assert()
 */
#ifdef NDEBUG
#define valgrind_assert(condition)
#define valgrind_assert_msg(condition,msg...)
#else
#define valgrind_assert(condition) \
    if (!(condition)) \
    { valgrind_assert_failed(__FILE__, __LINE__); }
#define valgrind_assert_msg(condition,msg...) \
    if (!(condition)) \
    { valgrind_assert_failed_msg(__FILE__, __LINE__, ## msg); }
#endif

/**
   Cause valgrind assertion error behavior w/o condition
 */
#define valgrind_fail(msg...) \
  valgrind_assert_failed_msg(__FILE__, __LINE__, ## msg);

#endif
