#ifndef __TRITON_THREAD_H__
#define __TRITON_THREAD_H__

#ifdef __TRITON_POSIX_LOCKING__
#include <pthread.h>

typedef pthread_mutex_t triton_mutex_t;
typedef pthread_mutexattr_t triton_mutexattr_t;
#define triton_mutex_lock(__mut) pthread_mutex_lock(__mut)
#define triton_mutex_unlock(__mut) pthread_mutex_unlock(__mut)
#define triton_mutex_trylock(__mut) pthread_mutex_trylock(__mut)
#define triton_mutex_init(__mut, __attr) pthread_mutex_init(__mut, __attr) 
#define triton_mutex_destroy(__mut) pthread_mutex_destroy(__mut)
#define TRITON_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
#define TRITON_COND_INITIALIZER PTHREAD_COND_INITIALIZER

typedef pthread_cond_t triton_cond_t;
typedef pthread_condattr_t triton_condattr_t;
#define triton_cond_init(__cond, __condattr) pthread_cond_init(__cond, __condattr)
#define triton_cond_signal(__cond) pthread_cond_signal(__cond)
#define triton_cond_broadcast(__cond) pthread_cond_broadcast(__cond)
#define triton_cond_wait(__cond, __mutex) pthread_cond_wait(__cond, __mutex)
#define triton_cond_timedwait(__cond, __mutex, __time) pthread_cond_timedwait(__cond, __mutex, __time)
#define triton_cond_destroy(__cond) pthread_cond_destroy(__cond)

#else /* __TRITON_POSIX_LOCKING__ */

/* for timespec struct def */
#include <time.h>

typedef int triton_mutex_t;
typedef int triton_mutexattr_t;
static inline int triton_mutex_lock(triton_mutex_t *__mut) { return 0; }
static inline int triton_mutex_unlock(triton_mutex_t *__mut) { return 0; }
static inline int triton_mutex_trylock(triton_mutex_t *__mut) { return 0; }
static inline int triton_mutex_init(triton_mutex_t *__mut, triton_mutexattr_t *__attr) {return 0; }
static inline int triton_mutex_destroy(triton_mutex_t *__mut) {return 0; }
#define TRITON_MUTEX_INITIALIZER 0

typedef int triton_cond_t;
typedef int triton_condattr_t;
#define TRITON_COND_INITIALIZER 0

static inline int triton_cond_init(triton_cond_t *__cond, triton_condattr_t *__condattr) { return 0; }
static inline int triton_cond_signal(triton_cond_t *__cond) { return 0; }
static inline int triton_cond_broadcast(triton_cond_t *__cond) { return 0; }
static inline int triton_cond_wait(triton_cond_t  *__cond, triton_mutex_t *__mutex) { return 0; }
static inline int triton_cond_timedwait(
    triton_cond_t *__cond, triton_mutex_t *__mutex, struct timespec *abstime) { return 0; }
static inline int triton_cond_destroy(triton_cond_t *__cond) { return 0; }

#endif /* __TRITON_POSIX_LOCKING__ */

#endif /* __TRITON_THREAD_H__ */
