#include "workqueue.h"
#include "triton-list.h"
#include <assert.h>
#include "triton-thread.h"
#include <stdlib.h>

typedef struct
{
   triton_list_link_t     link;
   workqueue_function_t * function;
   void *                 data;
} workentry_t;

struct workqueue_t
{
   triton_mutex_t lock;
   triton_list_t  list;
};

int workqueue_create (workqueue_t * q)
{
   triton_mutex_init (&q->lock, 0);
   triton_list_init (&q->list);
   return 1;
}

int workqueue_destroy (workqueue_t * q)
{
   triton_mutex_lock (&q->lock);
   assert (triton_list_empty (&q->list));
   triton_list_destroy (&q->list);
   triton_mutex_unlock (&q->lock);
   triton_mutex_destroy (&q->lock);
   return 1;
}


int workqueue_push (workqueue_t * q, workqueue_function_t * f, void * data)
{
   workentry_t * n = malloc (sizeof (workentry_t));
   n->function = f;
   n->data = data;

   triton_mutex_lock (&q->lock);
   triton_list_add_back (&n->link, &q->list);
   triton_mutex_unlock (&q->lock);
   return 1;
}

int workqueue_pop (workqueue_t * q, workqueue_function_t ** f, void ** data)
{
   workentry_t * n;
   int ret = 1;

   triton_mutex_lock (&q->lock);

   if (triton_list_empty (&q->list))
   {
      ret = 0;
   }
   else
   {
      n = triton_list_get_entry (triton_list_peek_front (&q->list),
            workentry_t, link);
      *f = n->function;
      *data = n->data;
      triton_list_del_front (&q->list);
   }

   triton_mutex_unlock (&q->lock);
   return ret;
}


